package Meopta;

my $pkg="Meopta";

use Exporter ();
@ISA=qw(Exporter);
@EXPORT=qw(encode_entities decode_entities encode_uri decode_uri printE printA);

use strict;
use POSIX qw(geteuid);

# funkce zjistuje uzivatele a heslo k SQL - bezpecna funkce pro suid scripty bezici s pravy tvurce scriptu
sub sql_user_password {
   my $uid=geteuid;
   my @passwd=getpwuid $uid;
   open(FILE,"$passwd[7]/.my.cnf") || die "nenalezen soubor $passwd[7]/.my.cnf";
   my ($user,$password);
   while(my $radek=<FILE>) {
      chomp($radek);
      if($radek=~/^[\t ]*user[\t ]*=[\t ]*([^\t ]*)$/) {
         $user=$1;
      }
      elsif($radek=~/^[\t ]*password[\t ]*=[\t ]*([^\t ]*)$/) {
         $password=$1;
      }
   }
   close(FILE);
   return ($user,$password);
}

# funkce zajisti spojeni s databazi, pokud neexistuje
my $dbh_script;
sub dbh {
   if(!defined($dbh_script)) {
      require DBI;
      my ($user,$pass)=$pkg->sql_user_password;
      my $driver="mysql";
      my $database="test";
      my $hostname="localhost";
      $dbh_script=DBI->connect("DBI:$driver:$database:$hostname",$user,$pass);
   }
   return $dbh_script;
}

# funkce zajisti odpojeni s databazi po skonceni scriptu
END {
   if(defined($dbh_script)) {
      $dbh_script->disconnect;
   }
}

# funkce, ktera prevadi nektere spec. znaky na jejich opis (entities)
sub encode_entities {
   my ($str)=@_;
   my %char2entity = (
      '&' => '&#38;',
      '<' => '&#60;',
      '>' => '&#62;',
      '"' => '&#34;',
      '|' => '&#124;'
   );
   $str=~s/([\<\&\>\"])/$char2entity{$1}/g;
   return $str;
}
sub decode_entities {
   my ($str)=@_;
   use HTML::Entities ();
   $str=HTML::Entities::decode($str);
   return $str;
}

# funkce, ktera prevadi vsechny znaky mimo pismen anglicke abecedy, cisel a znaku / . : _ na URI opis - vhodne pri generaci URI adresy
# nutne pro hodnoty URI atributu: href, src, action a take: background, classid, codebase, cite, data, datasrc, for (ve znacce <script>), longdesc, profile, usemap
# na prislusne atributy je pak nutne jeste aplikovat encode_entities, za predpokladu, ze obsahuji znak &, ktery nebyl umislne uri encodovan ...
sub encode_uri {
   my ($str)=@_;
   use URI::Escape;
   $str=uri_escape($str,'^A-Za-z0-9\/\.\:\_'); # zde musi byt apostrofy
   return $str;
}
sub decode_uri {
   my ($str)=@_;
   use URI::Escape;
   $str=uri_unescape($str);
   return $str;
}

# funkce slouzici k tisknuti hodnot na html strance -> je provadeno encode_entities
# funkce je implicitne exportovana - jeji volani pak vypada treba takto printE "pokus";
sub printE {
   my ($str)=@_;
   print encode_entities($str);
}

# funkce slouzici k tisknuti atributu HTML znacek
sub printA {
   my ($strA,$strV)=@_;
   print " $strA=\"";
   print encode_entities($strV);
   print "\"";
}

# funkce zjistuje zda ma uzivatel pro zadanou org na zadany okruh zadane pravo
sub pravo {
   my ($pkg_p,$user,$org,$okruh,$pravo)=@_;
   my $ok;
   my @user_list=($user,"*");
   if($user eq "*") {
      @user_list=("*");
   }
   my @org_list=($org,"*");
   if($org eq "*") {
      @org_list=("*");
   }
   my $dbh=$pkg->dbh;
   my $query_string_prava="
      select
         Prava.povoleni,
         Prava.blokace
      from
         Prava.Prava
      where
         Prava.user=? and
         Prava.org=? and
         Prava.okruh=? and
         Prava.pravo=?
   ";
   my $sth_prava=$dbh->prepare($query_string_prava);
   my $query_string_okruhy="
      select
         Okruhy.dedicnost,
         Okruhy.blokace
      from
         Prava.Okruhy
      where
         Okruhy.okruh=?
   ";
   my $sth_okruhy=$dbh->prepare($query_string_okruhy);
   my $okruhy_nalezen=0;
   while(1) {
      # zjisteni zda neni okruh zablokovan a zda se prava dedi z rodice
      $sth_okruhy->execute($okruh);
      if($sth_okruhy->rows) {
         $okruhy_nalezen=1;
      }
      my $ref_okruhy=$sth_okruhy->fetchrow_hashref;
      $sth_okruhy->finish;
      # pokud je okruh zablokovan => pravo neni
      if($$ref_okruhy{blokace} eq "1") {
         $ok=0;
         return $ok;
      }
      # zjisteni prava na okruhu
      foreach my $user (@user_list) {
         foreach my $org (@org_list) {
            $sth_prava->execute($user,$org,$okruh,$pravo);
            if($sth_prava->rows) {
               my $ref_prava=$sth_prava->fetchrow_hashref;
               $sth_prava->finish;
               if($$ref_prava{povoleni} eq "1" and $$ref_prava{blokace} eq "0") {
                  $ok=1;
               }
               else {
                  $ok=0;
               }
            }
            if(defined($ok)) {
               return $ok;
            }
         }
      }
      # prosel jsi celym stromem okruhu a nenasel zadny zaznam, ale nebyl take definovan zadny zaznam v Okruhy a je vyzadovano pravo X (proste jde o vpusteni do adresare na ktery neni definovano zadne omezeni ani na jeho rodice) => pravo je
      if($okruh eq "" and $pravo eq "X" and !$okruhy_nalezen) {
         $ok=1;
         return $ok;
      }
      # prosel jsi celym stromem okruhu a nenasel zadny zaznam => pravo neni
      if($okruh eq "") {
         $ok=0;
         return $ok;
      }
      # pokud prava na okruh se nededi od rodice, zkonci prochazeni stromem okruhu
      if($$ref_okruhy{dedicnost} eq "0") {
         $ok=0;
         return $ok;
      }
      # pravo jsi nenasel, jdi ve strome okruhu ke koreni a zkus to znovu
      if(rindex($okruh,"/")>=0) {
         $okruh=substr($okruh,0,rindex($okruh,"/"));
      }
      else {
         $okruh="";
      }
   }
   return $ok;
}

return 1;
